﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
  
namespace GSensor
{
    public partial class GSenor : Form
    {

        myGSensor myGSensor1 = new myGSensor();
        myGSensor.SensorData mySensorData = new myGSensor.SensorData();
        myGSensor.SensorDataOld mySensorDataOld = new myGSensor.SensorDataOld();
        
        //This value can be used to adjust how sensitive a scale you want to use
        //max is this value times the slider bars max (10)
        int scalevalue = 150;
        
        public GSenor()
        {
            InitializeComponent();

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            
           
           
        }


        private void timer1_Tick(object sender, EventArgs e)
        {
            //Reset background color
            shaketxt.BackColor = Color.White;

            //Number of shake detections needed to trigger command
            int shakecounter = 3;

            //Number of consecutive no shakes between shake detections needed to reset shakecounter
            int noshakecounter = 2;

            //Get Sensor Date
            myGSensor1.GetSensorData(ref mySensorData);

            //Update screen with latest Sensor Data
            TiltX_txt.Text = mySensorData.TiltX.ToString();
            TiltY_txt.Text = mySensorData.TiltY.ToString();
            TiltZ_txt.Text = mySensorData.TiltZ.ToString();
            thetaX_txt.Text = mySensorData.AngleX.ToString();
            thetaY_txt.Text = mySensorData.AngleY.ToString();

            //Detect if the phone is being shook
            if (myGSensor1.DetectShake(mySensorDataOld, mySensorData, threshold_trackBar.Value * scalevalue, shakecounter, noshakecounter))
            {
                shaketxt.BackColor = Color.Red;
                byte VK_RIGHT = 0x27;
                if(LaunchBox.Checked)
                  myGSensor1.KeyCommandForApplication("wmplayer.exe", VK_RIGHT);
            }
            
            //Update screen with additional information
            Counttxt.Text = myGSensor1.DetectWindowCount.ToString();
            NonCounttxt.Text = myGSensor1.ResetWindowCount.ToString();

            //Log the results when needed
            if(EventLogging.Checked)
              myGSensor1.Logger("gsensorlog.txt", mySensorData.TiltX, mySensorData.TiltY, mySensorData.TiltZ, mySensorData.AngleX, mySensorData.AngleY);

        }


    }

    public class myGSensor
    {
        public int DetectWindowCount = 0;
        public int ResetWindowCount = 0;
        int FirstTimeEntryFlag = 0;

        public struct SensorData
        {
            public short TiltX;          // From -1000 to 1000, 0 is flat
            public short TiltY;          // From -1000 to 1000, 0 is flat
            public short TiltZ;          // From -1000 to 1000, 0 = Straight up, -1000 = Flat, 1000 = Upside down
            public short Unknown1;
            public int AngleY;           // From 0 to 359 degrees
            public int AngleX;           // From 0 to 359 degrees
            public int Unknown2;
        };

        public struct SensorDataOld
        {
            public short TiltX;          // From -1000 to 1000, 0 is flat
            public short TiltY;          // From -1000 to 1000, 0 is flat
            public short TiltZ;          // From -1000 to 1000, 0 = Straight up, -1000 = Flat, 1000 = Upside down
        };

        #region HTCSensorSDK

        [DllImport("HTCSensorSDK")]
        extern static IntPtr HTCSensorGetDataOutput(IntPtr handle, out SensorData sensorData);

        [DllImport("HTCSensorSDK")]
        extern static IntPtr HTCSensorOpen(int sensor);

        [DllImport("HTCSensorSDK")]
        extern static void HTCSensorClose(IntPtr handle);

        [DllImport("coredll", SetLastError = true)]
        extern static IntPtr CreateEvent(IntPtr eventAttributes, bool manualReset, bool intialState, string name);

        [DllImport("coredll", SetLastError = true)]
        extern static bool EventModify(IntPtr handle, uint func);

        [DllImport("coredll")]
        extern static bool CloseHandle(IntPtr handle);

        // Get a handle to an application window.
        [DllImport("coredll.dll")] //Would be User32.dll on desktop
        private static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        // Activate an application window.
        [DllImport("coredll.dll")] //Would be User32.dll on desktop
        private static extern bool SetForegroundWindow(IntPtr hWnd);

        [DllImport("coredll", SetLastError = true)]
        private static extern void keybd_event(byte bVk, byte bScan, int dwFlags, int dwExtraInfo);


        #endregion

        public void GetSensorData(ref SensorData data)
        {
            //Initialise and start sensor
            IntPtr Handle = HTCSensorOpen(1);
            IntPtr hEvent = CreateEvent(IntPtr.Zero, true, false, "HTC_GSENSOR_SERVICESTART");
            EventModify(hEvent, 3);
            CloseHandle(hEvent);
            HTCSensorGetDataOutput(Handle, out data);

            return;
        }

        public bool DetectShake(SensorDataOld dataold, SensorData datanew, int threshold, int detectwindow, int resetwindow)
        {
            if (FirstTimeEntryFlag != 0)
            {
                //Convert values to use inbuilt Math library
                double Xold = Convert.ToDouble(dataold.TiltX); double Yold = Convert.ToDouble(dataold.TiltY);
                double Zold = Convert.ToDouble(dataold.TiltZ); double X = Convert.ToDouble(datanew.TiltX);
                double Y = Convert.ToDouble(datanew.TiltY); double Z = Convert.ToDouble(datanew.TiltZ);

                //Set new values to old
                dataold.TiltX = datanew.TiltX;
                dataold.TiltY = datanew.TiltY;
                dataold.TiltZ = datanew.TiltZ;

                //calculate Euclidean distance between old and data points
                double EuclideanDistance = Math.Sqrt(Math.Pow(X - Xold, 2) + Math.Pow(Y - Xold, 2) + Math.Pow(Y - Yold, 2));

                //set shake to true if distance between data points is greater than the defined threshold 
                if (EuclideanDistance > threshold)
                {
                    DetectWindowCount++;
                    if (DetectWindowCount > detectwindow) { DetectWindowCount = 0; ResetWindowCount = 0; return true; }
                }
                else
                {
                    ResetWindowCount++;
                    if (ResetWindowCount > resetwindow) { DetectWindowCount = 0; ResetWindowCount = 0; return false; }
                }

            }
            //no longer the first run.
            FirstTimeEntryFlag = 1;
            return false;
        }

        public void KeyCommandForApplication(string FileName, byte VK)
        {
            System.Diagnostics.Process p = new System.Diagnostics.Process();
            p.StartInfo.FileName = FileName;
            p.Start();
            const int KEYEVENTF_KEYUP = 0x02;
            const int KEYEVENTF_KEYDOWN = 0x00;
            keybd_event(VK, 0, KEYEVENTF_KEYDOWN, 0);
            keybd_event(VK, 0, KEYEVENTF_KEYUP, 0);
        }

        public void Logger(string FileName, int X, int Y, int Z, int AngleX, int AngleY)
        {
            TextWriter tw = new StreamWriter(FileName, true);
            tw.WriteLine(X.ToString() + "," + Y.ToString() + "," + Z.ToString() + "," + AngleX.ToString() + "," + AngleY.ToString());
            tw.Close();
        }
    }
}